%%%%%%%%%%%%%%%%%% Example 4.4 %%%%%%%%%%%%%%%%%%
%   Discrete-Time Control Problems using        %
%       MATLAB and the Control System Toolbox   %
%   by J.H. Chow, D.K. Frederick, & N.W. Chbat  %
%         Brooks/Cole Publishing Company        %
%                September 2002                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   ---- Impulse, step, and general response ----
%
clear
disp('Example 4.4')
Ts = 1;                            % sampling period (sec)
A = [0.8 -0.7 0.6; 1 0 0; 0 1 0];  % state-space matrices
B = [1; 0; 0];
C = [0 1.5 1];
D = 0;
G = ss(A,B,C,D,Ts)
disp('******>'), pause
dt = [0:Ts:50];                    % discrete-time vector


%----- (a) impulse response --------------
yi_G = impulse(G,dt);              % use time vector dt
figure
stem(dt,yi_G,':*'); grid           % plot with dotted stems
title('Impulse Response for Example 4.4')
xlabel('Time (s)')
ylabel('Amplitude')
disp('******>'), pause

%---- (b) step response --------------
ys_G = step(G,dt);                 % use time vector dt
figure
stem(dt,ys_G,':*'); grid           % plot with dotted stems
title('Step Response for Example 4.4')
xlabel('Time (s)')
ylabel('Amplitude')
disp('******>'), pause

%----(c) zero-state response to PWC input
u = 2*ones(size(dt));           % start with all points = 2.0
i = find(dt>=20);               % change values to 0.5 
u(i) = 0.5;                     % ...for dt >= 2.0
x0 = [0; 0; 0];                 % zero IC
[y_G,t] = lsim(G,u,[],x0);      % response to input with IC = 0
figure
plot(t,y_G,'*'),grid,hold on    % plot output with *'s, then
plot(t,u,'o'),hold off          % add input with o's
title('Zero-state response to PWC input for Example 4.4')
xlabel('Time (s)')
ylabel('Amplitude')
text(12,2.6,'input')
text(23,13,'output')
disp('******>'), pause

%---- (d) response to PWC input and nonzero IC
x0 = [1; 0; 2];                 % nonzero initial state
[y_G,t] = lsim(G,u,[],x0);      % response to input and IC
figure
plot(t,y_G,'*'),grid,hold on    % plot output with *'s, then
plot(t,u,'o'),hold off          % ... add input
xlabel('Time (s)')
ylabel('Amplitude')
text(12,2.6,'input')
text(23,13,'output')
title('Response to PWC input & nonzero IC for Example 4.4')
%%%%%%%%%%










