%%%%%%%%%%%%%%%%%% Example 5.8 %%%%%%%%%%%%%%%%%%
%   Discrete-Time Control Problems using        %
%       MATLAB and the Control System Toolbox   %
%   by J.H. Chow, D.K. Frederick, & N.W. Chbat  %
%         Brooks/Cole Publishing Company        %
%                September 2002                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ---- Closed-loop sampled-data control system ----
%
clear
disp('Example 5.8')

Gp = tf(1,[1 1 0])                  % plant in continuous tf form
fs =10, Ts = 0.1                    % sampling frequency and period
disp('******>'), pause
Gz = c2d(Gp,Ts,'zoh')               % zero-order hold
G_cz = tf(5*[1 -0.9],[1 -0.7],Ts)   % controller in discrete tf form

% ------ Part a: with ideal sensor -----
Ta = feedback(Gz*G_cz,1,-1)         % CL transfer function
[mag_a,theta_a] = xy2p(pole(Ta));   % CL poles in polar form
disp('******>'), pause
k  = [0:1:60]';
ya = step(Ta,k*Ts);                 % step response

% ------ Part b: with slow sensor ------
H  = tf(10,[1 10])                  % sensor in continuous tf form
HGz = c2d(H*Gp,Ts,'zoh')            % discretize combined H & Gp
disp('******>'), pause
Tb = Gz*G_cz/(1+HGz*G_cz)           % CL transfer fucntion
Tb = minreal(Tb)                    % eliminate duplicate states
[mag_b,theta_b] = xy2p(pole(Tb));   % CL poles in polar form

figure
yb = step(Tb,k*Ts);                 % step response
plot(k*Ts,ya,'*',k*Ts,yb,'o');grid  % plot step responses
legend('unity feedback', 'slow sensor')   % add lengend
xlabel('Time (s)')
ylabel('Amplitude')
title('Example 5.8: ideal sensor (*), sensor with dynamics (O)')
%%%%%%%%%%