%%%%%%%%%%%%%%%%% Example 8.3(d) %%%%%%%%%%%%%%%%
%   Discrete-Time Control Problems using        %
%       MATLAB and the Control System Toolbox   %
%   by J.H. Chow, D.K. Frederick, & N.W. Chbat  %
%         Brooks/Cole Publishing Company        %
%                September 2002                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   ---- Proportional-Integral-Derivative Control ----
%
clear
disp('Example 8.3(d)')
 
Ts = 0.1;                           % sampling period
Gp_s = tf(4,conv([2 1],[0.5 1]));   % continuous process
H_s = tf(1,[0.05 1]);               % continuous sensor
Gp_z = c2d(Gp_s,Ts,'zoh');          % discrete plant (no sensor)
HGp_s = H_s*Gp_s                    % continous plant
HGp_z = c2d(HGp_s,Ts,'zoh')         % discrete plant, using ZOH option
 
KI = 0.5; 
KD = 0.45;
KP = 1.95;

%----- PID controller -------
Gc = KP*(tf(1,1,Ts) ...                      % prorportional 
        + tf([KI*Ts  0],[1 -1],Ts) ...       % integral
        + tf(KD*[1 -1],[Ts 0],Ts));          % derivative 

dtime = 0:Ts:8;

%-------- reference input - CL transfer fcn
Tref_z = Gp_z*Gc/(1+HGp_z*Gc);
Tref_z = minreal(Tref_z);           % remove pole-zero cancellations
y_ref = step(Tref_z,dtime);         % CL response to step input
ref = dcgain(Tref_z);

%------- disturbance input - CL transfer fcn
Tdist_z = Gp_z/(1+HGp_z*Gc)
Tdist_z = minreal(Tdist_z);         % remove pole-zero cancellations
y_dist = step(Tdist_z,dtime);

%-------- plot responses --------
figure 
hold on
plot(dtime,y_ref,'bo');grid
plot(dtime,y_dist,'r+')
hold off
axis([0 8 0 1.20001])
xlabel('Time (sec)')
ylabel('Amplitude')
title(['Example 8.3(d) - Responses to step reference (o) and ',...
       'disturbance(+) inputs'])
text('Units','norm','Pos',[0.5,0.77],'Str','reference')
text('Units','norm','Pos',[0.5,0.10],'Str','disturbance')
text('Units','norm','Pos',[0.8,0.50],'Str',['KP = ',num2str(KP)])
text('Units','norm','Pos',[0.8,0.40],'Str',['KI = ',num2str(KI)])
text('Units','norm','Pos',[0.8,0.30],'Str',['KD = ',num2str(KD)])

%---- steady state values...Note that 'end' denotes last element
y_ref_ss = y_ref(end)
y_dis_ss = y_dist(end)
%%%%%%%%%%