%%%%%%%%% Comprehensive Problem dpower.m %%%%%%%%
%   Discrete-Time Control Problems using        %
%       MATLAB and the Control System Toolbox   %
%   by J.H. Chow, D.K. Frederick, & N.W. Chbat  %
%         Brooks/Cole Publishing Company        %
%                September 2002                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Electric Power Generation System
% This M-file contains the model of the single-machine 
% infinite-bus system described in Appendix A.
% The system has 7 states, 1 input, and 3 outputs. 
% The states of the model are:
%    x1 - rotor angle
%    x2 - rotor velocity
%    x3 - direct-axis voltage
%    x4 - direct-axis flux
%    x5 - quadrature-axis voltage
%    x6 - quadrature-axis flux
%    x7 - exciter state
%
% See also: dpower2

A = [       0   3.7699e+02            0            0 ...
            0            0            0;
  -2.4617e-01  -1.5633e-01  -1.3687e-01  -1.2318e-01 ...
  -1.2401e-02  -5.4566e-02            0;
   1.0922e-01   2.6271e-01  -2.1671e+00   2.3009e+00 ...
  -1.7068e-02  -7.5289e-02   1.2715e+00;
  -4.5840e+00            0   2.9954e+01  -3.4332e+01 ...
            0            0            0;
  -1.6060e-01            0            0            0 ...
  -8.4387e+00   6.3263e+00            0;
  -1.7004e+00            0            0            0 ...
   1.5222e+01  -2.1546e+01            0;
  -3.3926e+01  -2.3137e+01   6.8627e+00  -5.9469e+01 ...
   1.5038e+00   6.6335e+00  -1.1439e+02];

B = [ 0; 0; 0; 0; 0; 0; 16.4];

C = [  -0.1226    1.0500    0.2304    0.2074  ...
       -0.1045   -0.4599         0;
          0         1         0         0     ...
          0         0            0; 
	1.4158    0.9000    0.7871    0.7084  ...
        0.0713    0.3138   -0.0000];

D = zeros(3,1);

Gs = ss(A,B,C,D);
[num1,den] = tfdata(Gs(1,1),'v');
num1 = small20(num1,1e-10);
[num2,den] = tfdata(Gs(2,1),'v');
num2 = small20(num2,1e-10);
[num3,den] = tfdata(Gs(3,1),'v');
num3 = small20(num3,1e-10);

disp('Power system continuous-time state-space model in arrays A B C D')
disp('Power system continuous-time state-space model in LTI object Gs')
disp('  outputs are Vterm, w(speed), P')
disp('Power system transfer function')
disp('  den is denominator')
disp('  num1 is numerator from u to Vterm')
disp('  num2 is numerator from u to w(speed)')
disp('  num3 is numerator from u to P')
disp(' ')

Ts = 0.01;
Gz = c2d(Gs,Ts,'zoh');
[Ad,Bd,Cd,Dd] = ssdata(Gz);
[dnum1,dden] = tfdata(Gz(1,1),'v');
[dnum2,dden] = tfdata(Gz(2,1),'v');
[dnum3,dden] = tfdata(Gz(3,1),'v');

disp('Power system discretized model obtained using a sampling')
disp('  time of 0.01 sec and a zero-order hold')
disp('Discrete-time state-space model in arrays Ad Bd Cd Dd')
disp('Discrete-time state-space model in LTI object Gz')
disp('  outputs are Vterm, w(speed), P')
disp('Power system transfer function')
disp('  dden is denominator')
disp('  dnum1 is numerator from u to Vterm')
disp('  dnum2 is numerator from u to w(speed)')
disp('  dnum3 is numerator from u to P')

disp('The system data is saved in power.mat')

save power.mat A B C D Gs den num1 num2 num3 ...
               Ad Bd Cd Dd Gz dden dnum1 dnum2 dnum3 
%%%%%%%%%%