%%%%%%%%% Comprehensive Problem dstick.m %%%%%%%%
%   Discrete-Time Control Problems using        %
%       MATLAB and the Control System Toolbox   %
%   by J.H. Chow, D.K. Frederick, & N.W. Chbat  %
%         Brooks/Cole Publishing Company        %
%                September 2002                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inverted Pendulum
% This M-file contains the model of the open-loop inverted 
% pendulum described in Appendix A. The parameters of the 
% system are from an experiment made by Quanser Consulting, Inc. 
% The system has 4 states, 1 input, and 2 outputs. 
%   states are: 
%       x1 = stick angle (rad)
%		x2 = stick angular velocity (rad/s)
%		x3 = cart position  (feet)
%		x4 = cart velocity (ft/s)
%   input is: 
%     u = voltage applied to motor (volts)
%   outputs are: 
%   theta = stick angle
%   x = cart position

mp = 0.21;     % kg    - mass of pendulum
mc = 0.455;    % kg    - mass of cart
Ip = 0.305;    % m     - center of gravity
R  = 2.6;      % ohm   - armature resistance
K  = 4.47;     % N/A   - back emf constant
g  = 9.8;      % m/s^2 - gravity constant

%------- state-space matrices for pendulum
A = [  0               1  0    0          ;
    (mp+mc)*g/(mc*Ip)  0  0  K^2/(mc*Ip*R);
       0               0  0    1          ;
     -mp*g/mc          0  0  -K^2/(mc*R) ];
B = [ 0; -K/(mc*Ip*R); 0; K/(mc*R)];
C = [1 0 0 0;
     0 0 1 0];
D = zeros(2,1);
Gs = ss(A,B,C,D);
Gstf = tf(Gs);
[nump,denp] = tfdata(Gs(1,1),'v');
nump = small20(nump,1e-10);
[numc,denc] = tfdata(Gs(2,1),'v');
numc = small20(numc,1e-10);

disp('Inverted pendulum system continuous-time state-space model')
disp('  in arrays A B C D')
disp('Inverted pendulum system continuous-time state-space model')
disp('  in LTI object Gs')
disp('Inverted pendulum system continuous-time TF model')
disp('  in LTI object Gstf')
disp('  outputs are pendulum angle and cart position')
disp('Inverted pendulum system transfer function')
disp('  denp is denominator and nump is numerator')
disp('    from u to pendulum angle')
disp('  denc is denominator and numc is numerator')
disp('    from u to cart position')
disp(' ')

Ts = 0.01;   % sampling at 100 Hz
Gz = c2d(Gs,Ts,'zoh');
[Ad,Bd,Cd,Dd] = ssdata(Gz);
Gztf = tf(Gz);
[dnump,ddenp] = tfdata(Gz(1,1),'v');
[dnumc,ddenc] = tfdata(Gz(2,1),'v');

disp('Inverted pendulum system discretized model obtained using a sampling')
disp('  time of 0.01 sec and a zero-order hold')
disp('Discrete-time state-space model in arrays Ad Bd Cd Dd')
disp('Discrete-time state-space model in LTI object Gz')
disp('Discrete-time tf model in LTI object Gz')
disp('  outputs are pendulum angle and cart position')
disp('Inverted pendulum system transfer function')
disp('  ddenp is denominator and dnump is numerator')
disp('    from u to pendulum angle')
disp('  ddenc is denominator and dnumc is numerator')
disp('    from u to cart position')

disp('The system data are saved in stick.mat')

save stick.mat A B C D Gs Gstf denp nump denc numc ...
               Ad Bd Cd Dd Gz Gztf ddenp dnump ddenc dnumc 
%%%%%%%%%%